package indep_set;

import indep_set.compound.Calculation;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;


public class Config {
	
	public static final String CONFIG_FILE_NAME = "config.txt";
	
	public static int Delta;
	public static boolean root_node_min = true;
	public static boolean root_node_max = false;
	public static boolean exceptions_only = false;
	
	static final String DELTA_LABEL = "Delta";
	static final String ROOT_NODE_MIN_LABEL = "root_node_has_minimum_degree";
	static final String ROOT_NODE_MAX_LABEL = "root_node_has_maximum_degree";
	static final String PRINT_EXCEPTIONS_ONLY_LABEL = "print_exceptions_only";
	
	static {
		try{
			readConfig();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		Calculation.init();
	}
	
	static private void readConfig() throws IOException {
		BufferedReader reader = null;
		try{
			reader = new BufferedReader(new FileReader(CONFIG_FILE_NAME));
			String l;
			while ((l = reader.readLine()) != null) {
				interpretConfigLine(l);
            }
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		} finally{
			if (reader != null)
				reader.close();
		}
	}
	
	static private void interpretConfigLine(String line){
		if (line.startsWith(DELTA_LABEL)){
			Delta = Integer.valueOf(line.split("=")[1].trim());
		} else if (line.startsWith(ROOT_NODE_MIN_LABEL)){
			root_node_min = Boolean.valueOf(line.split("=")[1].trim());
		} else if (line.startsWith(ROOT_NODE_MAX_LABEL)){
			root_node_max = Boolean.valueOf(line.split("=")[1].trim());
		} else if (line.startsWith(PRINT_EXCEPTIONS_ONLY_LABEL)){
			exceptions_only = Boolean.valueOf(line.split("=")[1].trim());
		}
	}
	
	public static void main(String[] args){
		System.out.println(Delta);
		System.out.println(root_node_min);
		System.out.println(root_node_max);
		System.out.println(exceptions_only);	
	}
	
	public static String configLaTeX(){
		StringBuilder sb = new StringBuilder();
		sb.append("Settings: \n\n");
		sb.append("$\\Delta = " + Delta + "$\n\n");
		if (root_node_min){
			sb.append("Root node chosen to have minimum degree.\n\n");
		} 
		if (root_node_max){
			sb.append("Root node chosen to have maximum degree.\n\n");
		}
		if (exceptions_only){
			sb.append("Only non-positive entries are printed.\n\n");
		}
		sb.append("\\newpage\n\n");
		return sb.toString();		
	}
	
	public static String configText(){
		StringBuilder sb = new StringBuilder();
		sb.append("%Settings: \n");
		sb.append("%Delta = " + Delta + "\n");
		if (root_node_min){
			sb.append("%Root node chosen to have minimum degree.\n");
		}
		if (root_node_max){
			sb.append("%Root node chosen to have maximum degree.\n");
		}
		if (exceptions_only){
			sb.append("%Only non-positive entries are printed.\n");
		}
		return sb.toString();		
	}
	

}
