package indep_set.compound;

import static java.lang.Math.pow;
import indep_set.Config;
import indep_set.root.Root;

/**
 * A compound is consists of a "root" (the level 0,1,2 vertices)
 * as well as the down-degrees of the level 2 vertices.
 */
public class Compound {
	
	public Root root;
	public int[] outDegree; //the outdegrees of the lvl2 vertices.
	
	double weight = -1, weight1 = -1, weight2 = -1;
	
	public Compound(Root root, int[] outDegree){
		this.root = root;
		this.outDegree = outDegree.clone();
		assert(root.lvl2size == outDegree.length);
	}	
	
	public boolean inequalitySatisfied(){
		return weight() >= weight1() + weight2();
	}
	
	
	public double weight(){
		if (weight!= -1) return weight;
		
		double tempWeight = 1;
		
		tempWeight = 1;
		//01-edges
		for (int i = 0; i < root.lvl1size; i++){
			tempWeight *= Calculation.f(degree(0,0), degree(1,i));
		}
		//12-edges
		for (int i = 0; i < root.lvl1size; i++){
			for (int j : root.level1.get(i)){
				tempWeight *= Calculation.f(degree(1,i), degree(2,j));
			}
		}
		//23-edges
		for (int j = 0; j < root.lvl2size; j++){
			if (Config.root_node_max){
				tempWeight *= pow(Calculation.f(degree(2,j), root.lvl1size), outDegree[j]);
			} else {
				tempWeight *= pow(Calculation.f(degree(2,j), Config.Delta), outDegree[j]);
			}
		}
		
		weight = tempWeight;
		
		return tempWeight;
	}
	
	
	/**
	 * Weight with just the root node removed.
	 */
	public double weight1(){
		if (weight1!= -1) return weight1;
		
		double tempWeight = 1;
		
		tempWeight = 1;
		//12-edges
		for (int i = 0; i < root.lvl1size; i++){
			if (degree(1,i)==1){ //isolated vertex
				tempWeight *= 2;
				continue;
			}
			for (int j : root.level1.get(i)){
				tempWeight *= Calculation.f(degree(1,i)-1, degree(2,j));
			}
		}
		//23-edges
		for (int j = 0; j < root.lvl2size; j++){
			tempWeight *= pow(Calculation.f3(degree(2,j)), outDegree[j]);
		}
		
		weight1 = tempWeight;
		
		return tempWeight;
	}
		
	/**
	 * Weight with the root node and all its neighbors removed;
	 */
	public double weight2(){
		if (weight2!= -1) return weight2;
		
		double tempWeight = 1;
		
		tempWeight = 1;
		
		//23-edges
		for (int j = 0; j < root.lvl2size; j++){
			if (outDegree[j]==0){ //isolated vertex
				tempWeight *= 2;
			} else{
				tempWeight *= pow(Calculation.f3(outDegree[j]), outDegree[j]);
			}
			
		}
		
		weight2 = tempWeight;
		
		return tempWeight;
	}
	
	
	private int degree(int level, int vertexIndex){
		if (level==0){
			return root.lvl1size;
		} else if(level==1){
			return root.lvl1outDegree(vertexIndex) + 1;
		} else if(level==2){
			return root.lvl2inDegree(vertexIndex) + outDegree[vertexIndex];
		} else if(level==3){
			if (Config.root_node_min){
				return Config.Delta;
			} else {
				return root.lvl1size;
			}
			
		} else {
			assert(false);
			return -1;
		}
	}
	
	
	
}
