package indep_set.gen_root_code;

import static indep_set.Config.Delta;

import indep_set.Config;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * Generator for all "root" configurations.
 */
public class AllRootCodes {
	
	int lvl1size;
//	public StringBuilder output = new StringBuilder();
	public PrintWriter writer = null;
	
	List<List<Integer>> currentJoinConfig;
	int[] degreeRemaining;
	int[] zeros;
	
	int minLvl1DownDegree;
	int maxLvl1DownDegree;
	
	
	public AllRootCodes(int lvl1size, PrintWriter writer){
		this.writer = writer;
		this.lvl1size = lvl1size;
		init();
	}
	
	private void init(){
		currentJoinConfig = new ArrayList<List<Integer>>();
		degreeRemaining = new int[lvl1size];
		zeros = new int[lvl1size];
		
		if (Config.root_node_min){
			minLvl1DownDegree = lvl1size - 1;
			
		} else {
			minLvl1DownDegree = 0;
			
		}
		
		if (Config.root_node_max){
			maxLvl1DownDegree = lvl1size - 1;
		} else {
			maxLvl1DownDegree = Delta - 1;
		}
		
		
		for (int i = 0; i < lvl1size; i++){
			degreeRemaining[i] = maxLvl1DownDegree;
			zeros[i] = 0;
		}
	}
	
	public void generateAllRootCodes(){
		processCurrentJoinConfig(); //no joins	
		for (int k = 2; k <= Delta & k <= lvl1size; k++){
			firstJoin(k);
		}
	}
	
	
	private void firstJoin(int firstJointSize){
		List<Integer> first = new ArrayList<Integer>(firstJointSize);
		for (int i = 1; i <= firstJointSize; i++)
			first.add(i);
		if (tryAddJoin(first)){
			processCurrentJoinConfig(); //one join
			secondJoin(firstJointSize);
			removeLastJoin();
		}
	}
	
	private void secondJoin(int firstJointSize){
		for (int k = 2; k <= firstJointSize; k++){
			//k = secondJoinSize
			for (int j = k; j >= k + firstJointSize - lvl1size && j >= 0; j--){
				//j = overlap with the first join
				List<Integer> second = new ArrayList<Integer>(k);
				for (int i = 1; i <= j; i++){
					second.add(i);
				}
				for (int i = 1; i<= k-j; i++){
					second.add(firstJointSize+i);
				}
				
				if (tryAddJoin(second)){
					processCurrentAndSearchNext();
					removeLastJoin();
				}
			}			
		}
	}
	
	private void processCurrentAndSearchNext(){
		processCurrentJoinConfig();
		searchNextLowerSize();
		searchNextCurrentSize();		
	}
	
	private void searchNextCurrentSize(){
		List<Integer> last = currentJoinConfig.get(currentJoinConfig.size()-1);
		for (List<Integer> subset : new AllSubsets(lvl1size, last.size(), last)){
			if (tryAddJoin(subset)){
				processCurrentAndSearchNext();
				removeLastJoin();
			}
		}
	}
	
	private void searchNextLowerSize(){
		int k = currentSize() - 1;
		if (k<= 1) return;
		for (List<Integer> subset : new AllSubsets(lvl1size, k)){
			if (tryAddJoin(subset)){
				processCurrentAndSearchNext();
				removeLastJoin();
			}
		}
	}
	
	
	private boolean tryAddJoin(List<Integer> newJoin){
		for (int a : newJoin){
			if (degreeRemaining[a-1] == 0)
				return false;
		}
		currentJoinConfig.add(newJoin);
		for (int a : newJoin){
			degreeRemaining[a-1]--;
		}
		return true;
	}
	
	private List<Integer> removeLastJoin(){
		List<Integer> removedJoin = currentJoinConfig.remove(currentJoinConfig.size()-1);
		for (int a : removedJoin){
			degreeRemaining[a-1]++;
		}
		return removedJoin;
	}
	
	private int currentSize(){
		return currentJoinConfig.get(currentJoinConfig.size()-1).size();
	}
		
	private void processCurrentJoinConfig(){
		
		String joinConfigString = joinConfigString();
		
		int [] minDegree = new int[lvl1size];
		int [] maxDegree  = new int[lvl1size];
		for (int i = 0; i < lvl1size; i++){
			if (Config.root_node_min){
				minDegree[i] = max(minLvl1DownDegree, maxLvl1DownDegree - degreeRemaining[i]);
			} else {
				minDegree[i] = maxLvl1DownDegree - degreeRemaining[i];
			}
			if (Config.root_node_max){
				
			}
			
			maxDegree[i] = maxLvl1DownDegree;
		}
		
		//get all degrees
		for (int [] x : new MultiIterable(minDegree, maxDegree)){
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < lvl1size; i++){
				sb.append(String.valueOf(x[i]));
			}
			sb.append(" ");
			sb.append(joinConfigString);
			output(sb.toString());
		}
	}
	
	private void output(String s){
		writer.println(s);
	}
	
	
	private String joinConfigString(){
		StringBuilder sb = new StringBuilder();
		for (List<Integer> join : currentJoinConfig){
			for(int a : join)
				sb.append(String.valueOf(a));
			sb.append(" ");
		}
		return sb.toString();
	}
	
	
	private int max(int a, int b){
		return (a > b) ? a : b;
	}
}
