package indep_set.gen_root_code;

import java.util.Iterator;


/**
 * Iterates through all vectors between two given vectors a and b.
 * Equivalent to nested for loops.
 */
public class MultiIterable implements Iterable<int[]>{

	int[] start;
	int[] end;
	public MultiIterable(int[] start, int[] end){
		this.start = start.clone();
		this.end = end.clone();
		assert(start.length == end.length);
	}
	
	
	@Override
	public Iterator<int[]> iterator() {
		return new MultiIterator(start, end);
	}
	
	
	public class MultiIterator implements Iterator<int[]>{

		int[] start;
		int[] end;
		int[] cur;
		int n;
		boolean hasNext = true;
		
		public MultiIterator(int[] start, int[] end){
			this.start = start;
			this.end = end;
			this.cur = start.clone();
			n = start.length;
			for (int i = 0; i < n; i++){
				if (start[i] > end[i]){
					hasNext = false;
					break;
				}
			}
		}
		
		@Override
		public boolean hasNext() {
			return hasNext;
		}

		@Override
		public int[] next() {
			int[] toReturn = cur.clone();
						
			for (int i = 0; i < n; i++){
				if (cur[i] < end[i]){
					cur[i]++;
					return toReturn;
				} else {
					cur[i] = start[i];
				}
			}
			hasNext = false;
			return toReturn;
		}

		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}
		
	}

}
